library ieee;
use ieee.std_logic_1164.all;

entity module_A is
    port (
        I_clk      : in  std_logic;
        I_rst      : in  std_logic;
        I_wave_sel : in  std_logic_vector(1 downto 0);
        I_middle   : in  std_logic;
        I_last     : in  std_logic;
        O_u_d      : out std_logic;
        O_sign_sel : out std_logic
        );
end module_A;

architecture behavioral of module_A is

    type state_type is (st_0_to_PI_2, st_PI_2_to_PI, st_PI_to_3_PI_2, st_3_PI_2_to_2_PI);
    signal state_reg : state_type := st_0_to_PI_2;

begin

    process (I_clk)
    begin
        if rising_edge(I_clk) then
            if I_rst = '1' then
                state_reg  <= st_0_to_PI_2;
                O_sign_sel <= '0';
            else
                case state_reg is
                    when st_0_to_PI_2 =>
                        if I_middle = '1' then
                            state_reg <= st_PI_2_to_PI;
                        end if;
                    when st_PI_2_to_PI =>
                        if I_last = '1' then
                            state_reg <= st_PI_to_3_PI_2;
                        end if;
                    when st_PI_to_3_PI_2 =>
                        if I_middle = '1' then
                            state_reg <= st_3_PI_2_to_2_PI;
                        end if;
                    when st_3_PI_2_to_2_PI =>
                        if I_last = '1' then
                            state_reg <= st_0_to_PI_2;
                        end if;
                end case;

                O_sign_sel <= '0' when state_reg = st_0_to_PI_2 or state_reg = st_PI_2_to_PI else '1';

            end if;
        end if;
    end process;

    O_u_d <= '1' when (I_wave_sel /= "11" and (state_reg = st_0_to_PI_2 or state_reg = st_PI_to_3_PI_2)) or (I_wave_sel = "11" and (state_reg = st_0_to_PI_2 or state_reg = st_PI_2_to_PI)) else
             '0';

end behavioral;
