library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

entity module_B is
    generic (
        G_MAX_VAL : natural := 25
        );
    port (
        I_clk    : in  std_logic;
        I_rst    : in  std_logic;
        I_u_d    : in  std_logic;
        O_val    : out std_logic_vector(integer(ceil(log2(real(G_MAX_VAL)))) - 1 downto 0);
        O_last   : out std_logic;
        O_middle : out std_logic
        );
end module_B;

architecture behavioral of module_B is

    signal SR_val_reg     : natural range 0 to G_MAX_VAL := 0;
    constant C_middle_val : natural                      := G_MAX_VAL/2;

begin

    process(I_clk)
    begin
        if rising_edge(I_clk) then
            if I_rst = '1' then
                SR_val_reg <= 0;
            else
                if I_u_d = '1' then
                    SR_val_reg <= SR_val_reg + 1;
                else
                    SR_val_reg <= SR_val_reg - 1;
                end if;
            end if;
        end if;
    end process;

    O_last   <= '1' when (I_u_d = '1' and SR_val_reg = G_MAX_VAL-1) or (I_u_d = '0' and SR_val_reg = 1)                else '0';
    O_middle <= '1' when (I_u_d = '1'and SR_val_reg = C_middle_val-1) or (I_u_d = '0' and SR_val_reg = C_middle_val+1) else '0';

    O_val <= std_logic_vector(to_unsigned(SR_val_reg, integer(ceil(log2(real(G_MAX_VAL))))));

end behavioral;
