library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;
use work.wave_package.all;

entity module_D is
    generic (
        G_N  : integer := 8;
        G_f0 : real    := 1.0;
        G_fs : real    := 100.0
        );
    port (
        I_clk      : in  std_logic;
        I_rst      : in  std_logic;
        I_addr     : in  std_logic_vector(integer(ceil(log2(real(natural(floor(G_fs/(4.0*G_f0))))))) - 1 downto 0);
        O_triangle : out std_logic_vector(G_N-1 downto 0)
        );
end module_D;

architecture behavioral of module_D is

    constant C_LUT_SIZE    : natural                                     := natural(floor(G_fs/(4.0*G_f0))) + 1;  -- number of samples in the LUT for phases ranging from 0 to PI/2 (inclusive)
    signal SR_lut          : lut_type(0 to C_LUT_SIZE-1)(G_N-1 downto 0) := triangle_table(C_LUT_SIZE, G_N, G_f0, G_fs);
    signal SR_triangle_out : signed(G_N-1 downto 0)                      := (others => '0');
    signal SC_sat_addr     : integer range 0 to C_LUT_SIZE - 1;

begin

    SC_sat_addr <= to_integer(unsigned(I_addr)) when unsigned(I_addr) < C_LUT_SIZE else C_LUT_SIZE - 1;

    process(I_clk)
    begin
        if rising_edge(I_clk) then
            if I_rst = '1' then
                SR_triangle_out <= (others => '0');
            else
                SR_triangle_out <= SR_lut(SC_sat_addr);
            end if;
        end if;
    end process;

    O_triangle <= std_logic_vector(SR_triangle_out);

end behavioral;
