library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;
use work.wave_package.all;

entity module_E is
    generic (
        G_N  : integer := 8;
        G_f0 : real    := 1.0;
        G_fs : real    := 100.0
        );
    port (
        I_clk       : in  std_logic;
        I_rst       : in  std_logic;
        I_addr      : in  std_logic_vector(integer(ceil(log2(real(natural(floor(G_fs/(2.0*G_f0))))))) - 1 downto 0);
        O_saw_tooth : out std_logic_vector(G_N-1 downto 0)
        );
end module_E;

architecture behavioral of module_E is

    constant C_LUT_SIZE     : natural                                     := natural(floor(G_fs/(2.0*G_f0))) + 1;  -- number of samples in the LUT for phases ranging from 0 to PI (inclusive)
    signal SR_lut           : lut_type(0 to C_LUT_SIZE-1)(G_N-1 downto 0) := saw_tooth_table(C_LUT_SIZE, G_N, G_f0, G_fs);
    signal SR_saw_tooth_out : signed(G_N-1 downto 0)                        := (others => '0');

begin

    process(I_clk)
    begin
        if rising_edge(I_clk) then
            if I_rst = '1' then
                SR_saw_tooth_out <= (others => '0');
            else
                SR_saw_tooth_out <= SR_lut(to_integer(unsigned(I_addr)));
            end if;
        end if;
    end process;

    O_saw_tooth <= std_logic_vector(SR_saw_tooth_out);

end behavioral;
