library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity tb_ADSR_module is
end tb_ADSR_module;

architecture behavior of tb_ADSR_module is

    -- Component Declaration for the Unit Under Test (UUT)
    component ADSR_module
        generic (
            N   : natural := 8;
            N_t : natural := 16;
            A_t : real    := 10.0;
            D_t : real    := 2.0;
            R_t : real    := 50.0;
            fs  : real    := 48000.0
            );
        port (
            clk  : in  std_logic;
            rst  : in  std_logic;
            key  : in  std_logic;
            gain : out std_logic_vector(N-1 downto 0)
            );
    end component;

    -- Signals for connecting to the UUT
    signal clk  : std_logic := '0';
    signal rst  : std_logic := '0';
    signal key  : std_logic := '0';
    signal gain : std_logic_vector(7 downto 0);

    -- Clock period definition
    constant clk_period : time := 20 ns;

begin

    -- Instantiate the Unit Under Test (UUT)
    uut : ADSR_module
        port map (
            clk  => clk,
            rst  => rst,
            key  => key,
            gain => gain
            );

    -- Clock process definitions
    clk_process : process
    begin
        clk <= '0';
        wait for clk_period/2;
        clk <= '1';
        wait for clk_period/2;
    end process;

    -- Stimulus process
    stim_proc : process
    begin
        -- hold reset state for 100 ns.
        rst <= '1';
        wait for 100 ns;
        rst <= '0';

        -- Test sequence
        -- Key press (attack phase)
        key <= '1';
        wait for 500 ns;

        -- Key release (release phase)
        key <= '0';
        wait for 5000 ns;

        -- Key press (attack phase)
        key <= '1';
        wait for 50000 ns;

        -- Key release (release phase)
        key <= '0';
        wait for 500 ns;

        -- Key press (attack phase)
        key <= '1';
        wait for 5000 ns;

        -- Key release (release phase)
        key <= '0';
        wait for 500 ns;

        -- End of test
        wait;
    end process;

end behavior;
