library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

entity tb_module_B is
-- Testbench has no ports
end tb_module_B;

architecture behavior of tb_module_B is

    -- Component declaration for the Unit Under Test (UUT)
    component module_B
        generic (
            G_MAX_VAL : natural := 25
            );
        port (
            I_clk    : in  std_logic;
            I_rst    : in  std_logic;
            I_u_d    : in  std_logic;
            O_val    : out std_logic_vector(integer(ceil(log2(real(G_MAX_VAL)))) - 1 downto 0);
            O_last   : out std_logic;
            O_middle : out std_logic
            );
    end component;

    -- Clock period constant
    constant C_clk_period : time := 10 ns;

    constant C_MAX_VAL : natural := 25;

    -- Signals to connect to UUT
    signal SR_clk    : std_logic := '0';
    signal SR_rst    : std_logic := '0';
    signal SR_u_d    : std_logic := '0';
    signal SC_val    : std_logic_vector(integer(ceil(log2(real(C_MAX_VAL)))) - 1 downto 0);
    signal SC_last   : std_logic;
    signal SC_middle : std_logic;

begin

    -- Instantiate the Unit Under Test (UUT)
    uut : module_B
        generic map (
            G_MAX_VAL => C_MAX_VAL
            )
        port map (
            I_clk    => SR_clk,
            I_rst    => SR_rst,
            I_u_d    => SR_u_d,
            O_val    => SC_val,
            O_last   => SC_last,
            O_middle => SC_middle
            );

    -- Clock process
    clk_process : process
    begin
        while true loop
            SR_clk <= '0';
            wait for C_clk_period / 2;
            SR_clk <= '1';
            wait for C_clk_period / 2;
        end loop;
    end process;

    -- Stimulus process
    stim_proc : process
    begin
        -- Reset the system
        SR_rst <= '1';
        wait for C_clk_period * 2;
        SR_rst <= '0';

        -- Test case 1: Count up
        SR_u_d <= '1';
        wait for C_clk_period * 30;

        -- Test case 2: Count down
        SR_u_d <= '0';
        wait for C_clk_period * 30;

        -- Test case 3: Reset during operation
        SR_rst <= '1';
        wait for C_clk_period * 2;
        SR_rst <= '0';
        wait for C_clk_period * 10;

        -- End simulation
        wait;
    end process;

end behavior;
