library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

entity tb_module_D is
-- Testbench has no ports
end tb_module_D;

architecture behavior of tb_module_D is

    -- Component declaration for the Unit Under Test (UUT)
    component module_D
        generic (
            G_N  : integer := 8;
            G_f0 : real    := 1.0;
            G_fs : real    := 100.0
            );
        port (
            I_clk      : in  std_logic;
            I_rst      : in  std_logic;
            I_addr     : in  std_logic_vector(integer(ceil(log2(real(natural(floor(G_fs/(4.0*G_f0))))))) - 1 downto 0);
            O_triangle : out std_logic_vector(G_N-1 downto 0)
            );
    end component;

    -- Clock period constant
    constant C_clk_period : time := 10 ns;

    constant C_N  : integer := 16;
    constant C_f0 : real    := 1.0;
    constant C_fs : real    := 100.0;

    -- Signals to connect to UUT
    signal SR_clk          : std_logic := '0';
    signal SR_rst          : std_logic := '0';
    signal SR_addr         : std_logic_vector(integer(ceil(log2(real(natural(floor(C_fs/(4.0*C_f0))))))) - 1 downto 0);
    signal SC_triangle_out : std_logic_vector(15 downto 0);
    signal SR_addrNatural  : natural   := 0;


begin

    -- Instantiate the Unit Under Test (UUT)
    uut : module_D
        generic map (
            G_N  => C_N,
            G_f0 => C_f0,
            G_fs => C_fs
            )
        port map (
            I_clk      => SR_clk,
            I_rst      => SR_rst,
            I_addr     => SR_addr,
            O_triangle => SC_triangle_out
            );

    -- Clock process
    clk_process : process
    begin
        while true loop
            SR_clk <= '0';
            wait for C_clk_period / 2;
            SR_clk <= '1';
            wait for C_clk_period / 2;
        end loop;
    end process;

    SR_addr <= std_logic_vector(to_unsigned(SR_addrNatural, integer(ceil(log2(real(natural(floor(C_fs/(4.0*C_f0)))))))));

    -- Stimulus process
    stim_proc : process
    begin
        -- Reset the system
        SR_rst <= '1';
        wait for C_clk_period * 2;
        SR_rst <= '0';

        -- Test case 1: Sweep through all addresses
        for i in 0 to natural(floor(C_fs/(4.0*C_f0))) loop
            SR_addrNatural <= i;
            wait for C_clk_period;
        end loop;

        -- Test case 2: Reset during operation
        SR_rst         <= '1';
        wait for C_clk_period * 2;
        SR_rst         <= '0';
        SR_addrNatural <= 0;
        wait for C_clk_period * 10;

        -- End simulation
        wait;
    end process;

end behavior;
