library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity tb_module_G is
end tb_module_G;

architecture testbench of tb_module_G is
    signal SR_din  : std_logic_vector(7 downto 0);
    signal SC_dout : std_logic_vector(7 downto 0);

    -- Instantiate the Unit Under Test (UUT)
    component module_G
        generic (
            G_N : natural := 24
            );
        port (
            I_din  : in  std_logic_vector(7 downto 0);
            O_dout : out std_logic_vector(7 downto 0)
            );
    end component;

begin

    -- Instantiate the UUT
    uut : module_G
        generic map (
            G_N => 8
            )
        port map (
            I_din  => SR_din,
            O_dout => SC_dout
            );

    -- Stimulus process
    stim_proc : process
    begin
        -- Test case 1
        SR_din <= "00000001";                                                    -- 1
        wait for 10 ns;
        assert SC_dout = "11111111" report "Test case 1 failed" severity error;  -- -1

        -- Test case 2
        SR_din <= "00000010";                                                    -- 2
        wait for 10 ns;
        assert SC_dout = "11111110" report "Test case 2 failed" severity error;  -- -2

        -- Test case 3
        SR_din <= "00000100";                                                    -- 4
        wait for 10 ns;
        assert SC_dout = "11111100" report "Test case 3 failed" severity error;  -- -4

        -- Test case 4
        SR_din <= "10000000";                                                    -- -128 (two's complement)
        wait for 10 ns;
        assert SC_dout = "01111111" report "Test case 4 failed" severity error;  -- -128 (two's complement)
        -- be careful: this test suggests that a clipping error of one LSB is accepted.

        -- Test case 5
        SR_din <= "11111111";                                                    -- -1 (two's complement)
        wait for 10 ns;
        assert SC_dout = "00000001" report "Test case 5 failed" severity error;  -- 1

        wait;
    end process;
end testbench;
