library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

entity tb_wave_generator is
end tb_wave_generator;

architecture behavior of tb_wave_generator is

    -- Constants for the testbench
    constant N  : integer := 8;
    constant f0 : real    := 1.0;
    constant fs : real    := 100.0;

    -- Signals for the testbench
    signal clk      : std_logic                    := '0';
    signal rst      : std_logic                    := '0';
    signal wave_sel : std_logic_vector(1 downto 0) := "00";
    signal wav      : std_logic_vector(N-1 downto 0);

    -- Clock period definition
    constant clk_period : time := 10 ns;

begin

    process
    begin
        wait for 3 us;
        wave_sel <= std_logic_vector(unsigned(wave_sel) + 1);
    end process;

    -- Instantiate the Unit Under Test (UUT)
    uut : entity work.wave_generator
        generic map (
            G_N  => N,
            G_f0 => f0,
            G_fs => fs
            )
        port map (
            I_clk      => clk,
            I_rst      => rst,
            I_wave_sel => wave_sel,
            O_wav      => wav
            );

    -- Clock process definitions
    clk_process : process
    begin
        while true loop
            clk <= '0';
            wait for clk_period / 2;
            clk <= '1';
            wait for clk_period / 2;
        end loop;
    end process;

    -- Stimulus process
    stim_proc : process
    begin
        -- Reset the system
        rst <= '1';
        wait for 20 ns;
        rst <= '0';

        -- Wait for some time to observe the output
        wait for 1000 ns;

        -- Finish simulation
        wait;
    end process;

end behavior;
